#!/usr/bin/perl
=head1 NAME

contentselection.pl - This program calculates and organizes the bi-grams of a given file.

=head1 SYNOPSIS

This document takes a single file and identifies the topics. 

=head1 USAGE

Usage: contentselection.pl [OPTIONS]

=head1 INPUT: filename, stop, cleanfile, cntfile, measure, statfile, topicsfile, X
=head2 --filename

The name of the document that is to be summarized

=head2 --stop

The stoplist

=head2 --cleanfile

The file that is being printed to

=head2 -- cntfile

The file that holds the bigrams with their values

=head2 --measure

File that is used to identify the frequency of bigrams [Default: tscore]

=head2 --statfile

Holds the bigram with its frequency, number of occurances, and score

=head2 --topicsfile

File that holds the bigrams in order based upon their frequency

=head2 --x

The number of bigrams that will be printed at the end of the program

=head2 --help 

The help menu

=head1 Optional Arguments:

Displays the quick summary of program options.

=head1 OUTPUT

Output is all of the topics and their score. 

=head1 SYSTEM REQUIREMENTS

=over

=item * Perl (version 5.8.5 or better) - http://www.perl.org

=item * N-gram Statistics Package - http://search.cpan.org/dist/Text-NSP/

=back

=head1 CONTACT US
     
  Please contact us at:
    
      Bridget T. McInnes: bthomson at vcu.edu 

=head1 AUTHOR

 Bridget T. McInnes, Virginia Commonwealth University
 Tyler M. Pickering
 Sydney M. Hogue, George Mason University
 Isaac L. Barron, George Mason University

=head1 COPYRIGHT

Copyright (c) 2015

 Bridget T. McInnes, Virginia Commonwealth University
 bthomson at vcu.edu
    
This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

=cut
use Getopt::Long;
#use warnings;
 
GetOptions("filename=s", "stop=s", "cleanfile=s", "cntfile=s", "measure=s", "statfile=s", "topicsfile=s", "X=s", "help");

my $filename = $opt_filename;
my $stop = $opt_stop;
my $cleanfile = $opt_cleanfile; 
my $cntfile = $opt_cntfile;
my $statfile = $opt_statfile; 
my $topicsfile = $opt_topicsfile;
my $help = $opt_help;

my $measure = "tscore";
if (defined $opt_measure) {
    $measure = $opt_measure;
}

my $X = 10; 
if(defined $opt_X) { 
    $X = $opt_X; 
}

#if help is defined, print out help
if( defined $opt_help ) {
    $opt_help = 1;
    &showHelp();
    exit;
}
#help menu
sub showHelp() {

    print "\nThis is a program that takes a text file and returns the\n";
    print "most frequently occuring bigrams and the sentences in which\n";
    print "they are located.\n\n";

    print "Usage: perl contentselection.pl [OPTIONS] --filename <TEXT FILE> --stop <STOPLIST FILE>\n";
    print "--cleanfile <CLEAN FILE> --cntfile <COUNT FILE> \n";
    print "--statfile <STAT FILE> --topics <TOPICS FILE>\n\n";

    print "OPTIONS: \n\n";

    print "--filename               The name of the document that is to be\n";
    print "                         summarized.\n\n";

    print "--stop                   The stoplist.\n\n";

    print "--cleanfile              The file that is being printed to.\n\n";

    print "--cntfile                The file that holds the bigrams\n";
    print "                         with their values.\n\n";
    
    print "--measure                File that is used to identify the\n";
    print "                         frequency of bigrams [Default: tscore\n\n";

    print "--statfile               Holds the bigram with its frequency,\n";
    print "                         number of occurances, and score.\n\n";

    print "--topics                 File that holds the bigrams in\n";
    print "                         order based upon their frequency.\n\n";

    print "--X                      The number of bigrams that will be\n";
    print "                         printed at the end of the program.\n\n";

    print "--help                   Prints the help menu statement.\n\n";
}

my %score = ();
my %hash = ();
#open users input (first text file/ the file that will be summarized)
open (FILE, $filename)||die "Could not open File ($filename)";
open(CLEANFILE,">$cleanfile")||die "Could not open cleanfile file ($cleanfile)\n";
# clean the file 
while (<FILE>) {
    chomp;

    my $line = $_;
    $line = lc($_);
    $line =~s/[\.\,\?\!\"\:\'\;\-\(\)\*\[\]\{\}\_]/ /g;
    $line =~s/^\s*//g;
    $line =~s/\s*$//g;
    $line =~s/\s+/ /g;
    
  print CLEANFILE "$line\n";
}
close FILE; 
close CLEANFILE;

#getting the bigrams based on nsp
#print "count.pl --stop $stop $cntfile $cleanfile\n";
system "count.pl --stop $stop $cntfile $cleanfile";

#calculating their association based on nsp
#print "statistic.pl $measure $statfile $cntfile\n";
system "statistic.pl $measure $statfile $cntfile";

my %topics = ();
#open the stat file were the data is stored / split data into a hash
open (FILE, $statfile)||die "Could not open ($statfile).";
while (<FILE>) {
    chomp;
    #getting bigrams and their statistics from the stat file
    my @ngrams = split/<>/,$_;
    my $word1 = shift @ngrams;
    my $word2 = shift @ngrams;
    my $stats = shift @ngrams;
    my @nums = split/\s+/,$stats;
    my $score = $nums[1];

    $topics{"$word1 $word2"} = $score;
}
#open topics file file that contains the bigrams
open (TOPICSFILE,">$topicsfile")||die "Could not open ($topicsfile).";

my $counter = 0; 
#loop throught the bigrams sorting based on score
foreach my $bigrams (sort {$topics{$b}<=>$topics{$a}} keys %topics){
    
    if($counter == $X) { last; }

    print TOPICSFILE "$bigrams $topics{$bigrams}\n";
    
    $counter++; 
}

close TOPICSFILE;
