#!/usr/bin/perl
=head1 NAME

infoordering..pl - This program outputs a sumary based on the topic information from the contentselection.pl program.

=head1 SYNOPSIS

This document prints out the summary. 

=head1 USAGE

Usage: perl infoordering.pl --topics <TOPICS FILE> --text <TEXT FILE>

=head1 INPUT: 

=head2 --text

The original text file of the document

=head2 --topics 

The original text file of the document

=head2 --help

The help menu

=head1 OUTPUT

Output is the summary  

=head1 SYSTEM REQUIREMENTS

=over

=item * Perl (version 5.8.5 or better) - http://www.perl.org

=item * Math::SparseVector -http://search.cpan.org/dist/Math-SparseVector

=back

=head1 CONTACT US
     
  Please contact us at:
    
      Bridget T. McInnes: bthomson at vcu.edu 

=head1 AUTHOR

 Bridget T. McInnes, Virginia Commonwealth University
 Tyler M. Pickering
 Sydney M. Hogue, George Mason University
 Isaac L. Barron, George Mason University

=head1 COPYRIGHT

Copyright (c) 2015

 Bridget T. McInnes, Virginia Commonwealth University
 bthomson at vcu.edu
    
This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

=cut
use Getopt::Long;
use Math::SparseVector;
use warnings;
use strict;

my $topics;
my $text;
my $opt_help;
#my $sentFile;
GetOptions ("topics=s" => \$topics,
	    "text=s"   => \$text,
            "help"     => \$opt_help)
	  #  "sentFile=s"  => \$sentFile)
    or die("files not found");

#  if help is defined, print out help
if( defined $opt_help ) {
    $opt_help = 1;
    &showHelp();
    exit;
}

my %score = ();
my %integer = ();
my $num = 0;
#open topics file with bi grams
open (FILE, $topics)||die "Could not open ($topics)";
while (<FILE>) {
    chomp;
    #getting the words in the bigram and their stats 
    my @tokens = split/\s+/,$_;
    my $word1 = shift @tokens;
    my $word2 = shift @tokens;
    my $stats = shift @tokens;
    
    #give each bigram a unique number
    $integer{"$word1 $word2"} = $num++;
    #give each bigram their score
    $score{"$word1 $word2"} = $stats;

}
close FILE;
#  open the file
open(FILE, $text) || die "Could not open file ($text)\n";

#  initialize container to hold sentences
my @sentences = (); 

#  initilize array to contain words for a single sentence
my @sent = (); 

#  loop through file and get sentences
while(<FILE>) {
    chomp; 

    #  split line on white space
    my @array = split/\s+/;
    
    #  for every word in the line
    foreach my $word (@array) { 
	
	#  if this is common word that ends in a period
	if($word=~/^(Mr|Mrs|Ms|Miss|Dr|)\.$/) { 
	    #  push the word on the sentence array 
	    push @sent, $word; 
	    #  continue on
	    next; 
	}
	
	#  if the word is the end of a sentence
	if($word=~/[\.\?\!]$/) { 
	    #  push the word on the sentence array
	    push @sent, $word; 
	    #  get sentence in string form
	    my $sentence= join " ", @sent; 
	    #  push sentence string on sentences array
	    push @sentences, $sentence;
	    #  reinitialize sentence array
	    @sent = (); 
	    #  continue on
	    next; 
	}
	
	#  word is just word -- push on sentence array
	push @sent, $word; 
    }
}

my %summary = (); 

#  instantiate topic vec
my $topicvec = Math::SparseVector->new;

#  load topic vec
foreach my $tpc (sort keys %integer) {
    $topicvec->set($integer{$tpc}, $score{$tpc}); 
}

#  normalize the vector
$topicvec->normalize; 

foreach my $sent (@sentences) {
    #to get the original version of each sentence
    my $dirtysentence = $sent;
    #  instantiate sentence vec
    my $sentvec = Math::SparseVector->new; 
    
    #cleaning the sentence
    $sent = lc($sent);
    $sent =~s/[\.\,\?\!\"\:\'\;\-\(\)\*\[\]\{\}\_]/ /g;
    $sent =~s/^\s*//g;
    $sent =~s/\s*$//g;
    $sent =~s/\s+/ /g;
    $sent =~s/ s / /g;

    #  load sentence vec
    foreach my $tpc (sort keys %score) { 
	if($sent=~/$tpc/) { 
	    $sentvec->set($integer{$tpc}, $score{$tpc});
	}
    }

    #  normalize the sentence vector
    $sentvec->normalize; 

    #  calculate the cosine
    my $dotprod = $sentvec->dot($topicvec);
    
    #  save the dot product in the summary hash
    $summary{$dirtysentence} = $dotprod;
    
    #  deallocate sentence vector
    $sentvec->free;
    
}

#  print out the top sentences to create the summary
my $count = 5; my $i = 0; 
foreach my $sent (sort keys %summary){
    print "$sent "; $i++; 
    if($i >= $count) { last; }
}
print "\n";

#Help Menu

sub showHelp() {

    print "\nThis is a program that takes a text file and returns the\n";
    print "most frequently occuring bigrams and the sentences in which\n";
    print "they are located.\n\n";

    print "Usage: perl infoordering.pl --topics <TOPICS FILE> --text <TEXT FILE>\n\n";

    print "OPTIONS: \n\n";

    print "--topics                 The wonderland.topics file that holds\n";
    print "                         the selection of bigrams.\n\n";

    print "--text                   The original text file of the document.\n\n";

    print "--help                   The help file.\n\n";
}
